@@ -870,7 +870,11 @@ def create_contrast_normalization_tasks(
     minval=None, maxval=None, bounds=None,
     bounds_mip=0
   ):
-
+  """
+  Use the output of luminence levels to contrast
+  correct the image by stretching the histogram
+  to cover the full range of the data type.
+  """
   srcvol = CloudVolume(src_path, mip=mip)
   
   try:
@@ -881,7 +885,10 @@ def create_contrast_normalization_tasks(
     dvol.info['scales'] = dvol.info['scales'][:mip+1]
     dvol.commit_info()
 
-  if shape == None:
+  if bounds is None:
+    bounds = srcvol.bounds.clone()
+
+  if shape is None:
     shape = Bbox( (0,0,0), (2048, 2048, 64) )
     shape = shape.shrink_to_chunk_size(dvol.underlying).size3()
     shape = Vec.clamp(shape, (1,1,1), bounds.size3() )
@@ -895,12 +902,11 @@ def create_contrast_normalization_tasks(
 
   class ContrastNormalizationTaskIterator(FinelyDividedTaskIterator):
     def task(self, shape, offset):
-      task_shape = min2(shape.clone(), srcvol.bounds.maxpt - offset)
       return ContrastNormalizationTask( 
         src_path=src_path, 
         dest_path=dest_path,
         levels_path=levels_path,
-        shape=task_shape, 
+        shape=shape.clone(), 
         offset=offset.clone(), 
         clip_fraction=clip_fraction,
         mip=mip,
@@ -935,10 +941,10 @@ def create_contrast_normalization_tasks(
   return ContrastNormalizationTaskIterator(bounds, shape)
 
 def create_luminance_levels_tasks(
-    layer_path, levels_path=None, coverage_factor=0.01, 
-    shape=None, offset=(0,0,0), mip=0, 
-    bounds_mip=0, bounds=None
-  ):
+  layer_path, levels_path=None, coverage_factor=0.01, 
+  shape=None, offset=None, mip=0, 
+  bounds_mip=0, bounds=None
+):
   """
   Compute per slice luminance level histogram and write them as
   $layer_path/levels/$z. Each z file looks like:
@@ -961,16 +967,32 @@ def create_luminance_levels_tasks(
     bounds_mip (int): mip of the input bounds
     bounds (Bbox-like)
   """
+  if shape or offset:
+    print(yellow(
+      "Create Luminance Levels Tasks: Deprecation Notice: "
+      "shape and offset parameters are deprecated in favor of the bounds argument."
+    ))
+
   vol = CloudVolume(layer_path, mip=mip)
 
-  if shape is None:
+  if bounds is None:
+    bounds = vol.bounds.clone()
+
+  bounds = get_bounds(vol, bounds, mip, bounds_mip=bounds_mip)
+
+  if shape is None and bounds is None:
     shape = Vec(*vol.shape)
     shape.z = 1
+  elif bounds is not None:
+    shape = Vec(*bounds.size3())
+    shape.z = 1
+
+  if not offset:
+    offset = bounds.minpt
 
   offset = Vec(*offset)
   zoffset = offset.clone()
 
-  bounds = get_bounds(vol, bounds, mip, bounds_mip=bounds_mip)
   protocol = vol.meta.path.protocol
 
   class LuminanceLevelsTaskIterator(object):
