@@ -56,7 +56,7 @@ class Tuple2(click.ParamType):
 
 @click.group()
 @click.option("-p", "--parallel", default=1, help="Run with this number of parallel processes. If 0, use number of cores.")
-@click.version_option(version="3.2.0")
+@click.version_option(version="4.0.0")
 @click.pass_context
 def main(ctx, parallel):
   """
@@ -92,7 +92,20 @@ def license():
   with open(path, 'rt') as f:
     print(f.read())
 
-@main.command()
+@main.group("image")
+def imagegroup():
+  """
+  Manipulate image volumes. (subgroup)
+
+  Images are the base datastructure in
+  Neuroglancer. This subgroup offers
+  methods for downsampling, transfers,
+  reencoding, rechunking, sharding,
+  and contrast correction.
+  """
+  pass
+
+@imagegroup.command()
 @click.argument("path")
 @click.option('--queue', default=None, required=True, help="AWS SQS queue or directory to be used for a task queue. e.g. sqs://my-queue or ./my-queue. See https://github.com/seung-lab/python-task-queue")
 @click.option('--mip', default=0, help="Build upward from this level of the image pyramid. Default: 0")
@@ -182,7 +195,7 @@ def downsample(
   tq = TaskQueue(normalize_path(queue))
   tq.insert(tasks, parallel=parallel)
 
-@main.command()
+@imagegroup.command()
 @click.argument("src")
 @click.argument("dest")
 @click.option('--queue', default=None, required=True, help="AWS SQS queue or directory to be used for a task queue. e.g. sqs://my-queue or ./my-queue. See https://github.com/seung-lab/python-task-queue")
@@ -214,7 +227,7 @@ def xfer(
   dest_voxel_offset, clean_info, no_src_update
 ):
   """
-  Transfer an image layer to another location.
+  Copy, re-encode, or shard an image layer.
 
   It is crucial to choose a good task shape. The task
   shape must be a multiple of two of the destination
@@ -267,6 +280,108 @@ def xfer(
   tq = TaskQueue(normalize_path(queue))
   tq.insert(tasks, parallel=parallel)
 
+@imagegroup.group("contrast")
+def contrastgroup():
+  """Perform contrast correction on the image."""
+  pass
+
+@contrastgroup.command()
+@click.argument("path")
+@click.option('--queue', default=None, required=True, help="AWS SQS queue or directory to be used for a task queue. e.g. sqs://my-queue or ./my-queue. See https://github.com/seung-lab/python-task-queue")
+@click.option('--mip', default=0, help="Build histogram from this level of the image pyramid.", show_default=True)
+@click.option('--coverage', default=0.01, type=float, help="Fraction of the image to sample. Range: [0,1]", show_default=True)
+@click.option('--xrange', type=Tuple2(), default=None, help="If specified, set x-bounds for sampling in terms of selected mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024.", show_default=True)
+@click.option('--yrange', type=Tuple2(), default=None, help="If specified, set y-bounds for sampling in terms of selected mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024", show_default=True)
+@click.option('--zrange', type=Tuple2(), default=None, help="If specified, set z-bounds for sampling in terms of selected mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1", show_default=True)
+@click.option('--bounds-mip', default=0, help="Build upward from this level of the image pyramid.", show_default=True)
+@click.pass_context
+def histogram(
+  ctx, path, queue, mip, coverage,
+  xrange, yrange, zrange, bounds_mip
+):
+  """(1) Compute the histogram for each z-slice."""
+  path = cloudfiles.paths.normalize(path)
+
+  bounds = None
+  if xrange or yrange or zrange:
+    bounds = CloudVolume(path).meta.bounds(mip)
+
+  if xrange:
+    bounds.minpt.x = xrange[0]
+    bounds.maxpt.x = xrange[1]
+  if yrange:
+    bounds.minpt.y = yrange[0]
+    bounds.maxpt.y = yrange[1]
+  if zrange:
+    bounds.minpt.z = zrange[0]
+    bounds.maxpt.z = zrange[1]
+
+  tasks = tc.create_luminance_levels_tasks(
+    path, 
+    levels_path=None, 
+    coverage_factor=coverage, 
+    mip=mip, 
+    bounds_mip=bounds_mip, 
+    bounds=bounds,
+  )
+
+  parallel = int(ctx.obj.get("parallel", 1))
+  tq = TaskQueue(normalize_path(queue))
+  tq.insert(tasks, parallel=parallel)
+
+@contrastgroup.command()
+@click.argument("src")
+@click.argument("dest")
+@click.option('--shape', default="2048,2048,64", type=Tuple3(), help="Size of individual tasks in voxels.", show_default=True)
+@click.option('--translate', type=Tuple3(), default=(0, 0, 0), help="Translate the bounding box by X,Y,Z voxels in the new location.")
+@click.option('--queue', default=None, required=True, help="AWS SQS queue or directory to be used for a task queue. e.g. sqs://my-queue or ./my-queue. See https://github.com/seung-lab/python-task-queue")
+@click.option('--mip', default=0, help="Apply normalization to this level of the image pyramid.", show_default=True)
+@click.option('--clip-fraction', default=0.01, type=float, help="Fraction of histogram on left and right sides to clip. Range: [0,1]", show_default=True)
+@click.option('--fill-missing', is_flag=True, default=False, help="Interpret missing image files as background instead of failing.")
+@click.option('--minval', default=None, help="Set left side of histogram as this value. (must be in range of datatype)", show_default=True)
+@click.option('--maxval', default=None, help="Set right side of histogram as this value. (must be in range of datatype)", show_default=True)
+@click.option('--xrange', type=Tuple2(), default=None, help="If specified, set x-bounds for sampling in terms of selected mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024.", show_default=True)
+@click.option('--yrange', type=Tuple2(), default=None, help="If specified, set y-bounds for sampling in terms of selected mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1024", show_default=True)
+@click.option('--zrange', type=Tuple2(), default=None, help="If specified, set z-bounds for sampling in terms of selected mip. By default the whole dataset is selected. The bounds must be chunk aligned to the task size e.g. 0,1", show_default=True)
+@click.option('--bounds-mip', default=0, help="Build upward from this level of the image pyramid.", show_default=True)
+@click.pass_context
+def equalize(
+  ctx, src, dest, queue,
+  shape, mip, clip_fraction,
+  fill_missing, translate,
+  minval, maxval,
+  xrange, yrange, zrange, bounds_mip
+):
+  """(2) Apply histogram equalization to z-slices."""
+  src = cloudfiles.paths.normalize(src)
+  dest = cloudfiles.paths.normalize(dest)
+
+  bounds = None
+  if xrange or yrange or zrange:
+    bounds = CloudVolume(path).meta.bounds(mip)
+
+  if xrange:
+    bounds.minpt.x = xrange[0]
+    bounds.maxpt.x = xrange[1]
+  if yrange:
+    bounds.minpt.y = yrange[0]
+    bounds.maxpt.y = yrange[1]
+  if zrange:
+    bounds.minpt.z = zrange[0]
+    bounds.maxpt.z = zrange[1]
+
+  tasks = tc.create_contrast_normalization_tasks(
+    src, dest, levels_path=None,
+    shape=shape, mip=mip, clip_fraction=clip_fraction,
+    fill_missing=fill_missing, translate=translate,
+    minval=minval, maxval=maxval, bounds=bounds,
+    bounds_mip=bounds_mip
+  )
+
+  parallel = int(ctx.obj.get("parallel", 1))
+  tq = TaskQueue(normalize_path(queue))
+  tq.insert(tasks, parallel=parallel)
+
 @main.command()
 @click.argument("queue", type=str)
 @click.option('--aws-region', default=SQS_REGION_NAME, help=f"AWS region in which the SQS queue resides.", show_default=True)
@@ -462,7 +577,7 @@ def mesh_merge(ctx, path, queue, magnitude, dir):
 @click.argument("path")
 @click.option('--queue', required=True, help="AWS SQS queue or directory to be used for a task queue. e.g. sqs://my-queue or ./my-queue. See https://github.com/seung-lab/python-task-queue", type=str)
 @click.option('--vqb', default=16, help="Vertex quantization bits. Can be 10 or 16.", type=int, show_default=True)
-@click.option('--compress-level', default=1, help="Draco compression level.", type=int, show_default=True)
+@click.option('--compress-level', default=7, help="Draco compression level.", type=int, show_default=True)
 @click.option('--shard-index-bytes', default=2**13, help="Size in bytes to make the shard index.", type=int, show_default=True)
 @click.option('--minishard-index-bytes', default=2**15, help="Size in bytes to make the minishard index.", type=int, show_default=True)
 @click.option('--minishard-index-encoding', default="gzip", help="Minishard indices can be compressed. gzip or raw.", show_default=True)
@@ -739,15 +854,7 @@ def skeleton_sharded_merge(
   tq = TaskQueue(normalize_path(queue))
   tq.insert(tasks, parallel=parallel)
 
-@main.group("rm")
-def deletegroup():
-  """
-  Parallelize the deletion process for
-  different kinds of data.
-  """
-  pass
-
-@deletegroup.command("image")
+@imagegroup.command("rm")
 @click.argument("path")
 @click.option('--queue', required=True, help="AWS SQS queue or directory to be used for a task queue. e.g. sqs://my-queue or ./my-queue. See https://github.com/seung-lab/python-task-queue", type=str)
 @click.option('--mip', default=0, help="Which mip level to start deleting from. Default: 0")
